@@ -2,6 +2,8 @@ from __future__ import unicode_literals, division, absolute_import, print_functi
 import atexit
 import codecs
 from contextlib import contextmanager
+import copy
+import fnmatch
 import signal
 import os
 import sys
@@ -9,6 +11,7 @@ import shutil
 import logging
 import threading
 import pkg_resources
+import Queue
 import yaml
 from datetime import datetime, timedelta
 
@@ -26,6 +29,8 @@ from flexget import config_schema, db_schema
 from flexget.event import fire_event
 from flexget.ipc import IPCServer, IPCClient
 from flexget.scheduler import Scheduler
+from flexget.task import Task
+from flexget.task_queue import TaskQueue
 from flexget.utils.tools import pid_exists
 
 log = logging.getLogger('manager')
@@ -114,6 +119,7 @@ class Manager(object):
 
         self.scheduler = Scheduler(self)
         self.ipc_server = IPCServer(self, options.ipc_port)
+        self.task_queue = TaskQueue()
         manager = self
         self.initialize()
 
@@ -163,7 +169,48 @@ class Manager(object):
     def has_lock(self):
         return self._has_lock
 
-    def run_cli_command(self):
+    def execute(self, options=None, output=None, priority=1):
+        """
+        Run all (can be limited with options) tasks from the config.
+
+        :param options: Either an :class:`argparse.Namespace` instance, or a dict, containing options for execution
+        :param output: If a file-like object is specified here, log messages and stdout from the execution will be
+            written to it.
+        :param priority: If there are other executions waiting to be run, they will be run in priority order,
+            lowest first.
+        :returns: a list of :class:`threading.Event` instances which will be
+            set when each respective task has finished running
+        """
+        if options is None:
+            options = copy.copy(self.options.execute)
+        elif isinstance(options, dict):
+            options_namespace = copy.copy(self.options.execute)
+            options_namespace.__dict__.update(options)
+            options = options_namespace
+        task_names = self.tasks
+        # Handle --tasks
+        if options.tasks:
+            # Create list of tasks to run, preserving order
+            task_names = []
+            for arg in options.tasks:
+                matches = [t for t in self.tasks if fnmatch.fnmatchcase(unicode(t).lower(), arg.lower())]
+                if not matches:
+                    log.error('`%s` does not match any tasks' % arg)
+                    continue
+                task_names.extend(m for m in matches if m not in task_names)
+            # Set the option as a list of matching task names so plugins can use it easily
+            options.tasks = task_names
+        # TODO: 1.2 This is a hack to make task priorities work still, not sure if it's the best one
+        task_names = sorted(task_names, key=lambda t: self.config['tasks'][t].get('priority', 65535))
+
+        finished_events = []
+        for task_name in task_names:
+            task = Task(self, task_name, options=options, output=output, priority=priority)
+            self.task_queue.put(task)
+            finished_events.append(task.finished_event)
+        return finished_events
+
+    def start(self):
         """
         Starting point when executing from commandline, dispatch execution
         to correct destination.
@@ -176,15 +223,17 @@ class Manager(object):
         command = self.options.cli_command
         options = getattr(self.options, command)
         # First check for built-in commands
-        if command == 'execute':
-            self.execute_command(options)
-        elif command == 'daemon':
-            self.daemon_command(options)
-        elif command == 'webui':
-            self.webui_command(options)
+        if command in ['execute', 'daemon', 'webui']:
+            if command == 'execute':
+                self.execute_command(options)
+            elif command == 'daemon':
+                self.daemon_command(options)
+            elif command == 'webui':
+                self.webui_command(options)
         else:
             # Otherwise dispatch the command to the callback function
             options.cli_command_callback(self, options)
+        self._shutdown()
 
     def execute_command(self, options):
         """
@@ -212,16 +261,11 @@ class Manager(object):
         # Otherwise we run the execution ourselves
         with self.acquire_lock():
             fire_event('manager.execute.started', self)
-            self.scheduler.start(run_schedules=False)
-            self.scheduler.execute(options)
-            self.scheduler.shutdown(finish_queue=True)
-            try:
-                self.scheduler.wait()
-            except KeyboardInterrupt:
-                log.error('Got ctrl-c exiting after this task completes. Press ctrl-c again to abort this task.')
-            else:
-                fire_event('manager.execute.completed', self)
-            self.shutdown(finish_queue=False)
+            self.task_queue.start()
+            self.execute(options)
+            self.shutdown(finish_queue=True)
+            self.task_queue.wait()
+            fire_event('manager.execute.completed', self)
 
     def daemon_command(self, options):
         """
@@ -243,14 +287,11 @@ class Manager(object):
                     # main thread, this error will occur.
                     log.debug('Error registering sigterm handler: %s' % e)
                 self.ipc_server.start()
-                fire_event('manager.daemon.started', self)
                 self.scheduler.start()
-                try:
-                    self.scheduler.wait()
-                except KeyboardInterrupt:
-                    log.info('Got ctrl-c, shutting down.')
-                    fire_event('manager.daemon.completed', self)
-                    self.shutdown(finish_queue=False)
+                fire_event('manager.daemon.started', self)
+                self.task_queue.start()
+                self.task_queue.wait()
+                fire_event('manager.daemon.completed', self)
         elif options.action == 'status':
             ipc_info = self.check_ipc_info()
             if ipc_info:
@@ -288,7 +329,10 @@ class Manager(object):
             self.daemonize()
         from flexget.ui import webui
         with self.acquire_lock():
+            self.ipc_server.start()
             webui.start(self)
+            self.task_queue.start()
+            self.task_queue.wait()
 
     def _handle_sigterm(self, signum, frame):
         log.info('Got SIGTERM. Shutting down.')
@@ -717,20 +761,14 @@ class Manager(object):
 
     def shutdown(self, finish_queue=True):
         """
-        Application is being exited
+        Request manager shutdown.
 
         :param bool finish_queue: Should scheduler finish the task queue
         """
-        # Wait for scheduler to finish
-        self.scheduler.shutdown(finish_queue=finish_queue)
-        try:
-            self.scheduler.wait()
-        except KeyboardInterrupt:
-            log.debug('Not waiting for scheduler shutdown due to ctrl-c')
-            # show real stack trace in debug mode
-            if manager.options.debug:
-                raise
-            print('**** Keyboard Interrupt ****')
+        self.task_queue.shutdown(finish_queue)
+
+    def _shutdown(self):
+        """Runs when the manager is done processing everything."""
         fire_event('manager.shutdown', self)
         if not self.unit_test:  # don't scroll "nosetests" summary results when logging is enabled
             log.debug('Shutting down')
